library saifu_qr;

import 'package:card_swiper/card_swiper.dart';
import 'package:flutter/material.dart';
import 'package:qr_flutter/qr_flutter.dart';

// ignore: must_be_immutable
class SaifuQR extends StatefulWidget {
  // Data [List or Object] recieved
  var data;

  // If enable, the package will split the data into chunks, or else, use your own methods
  final bool splitData;

  // The amount of characters, the data is to be split into, if splitData is enabled
  final int charPerFrame;

  // QRErrorCorrectLevel.H / M / S, higher levels ensures the qr can be read more correctly and less errors likely [more data is used]
  final int errorCorrectionalLevel;

  // This is the duration/speed of the transition from one qr to another (when multiple qr are displayed)
  final double transitionDuration;

  final double? fade;
  final double scale;

  final bool loop;
  final bool autoplay;
  final int autoplayDelay;
  final double viewportFraction;

  final Axis scrollDirection;
  final Color backgroundColor;
  final Color foregroundColor;
  final double borderRadius;
  final double qrPadding;

  final double itemWidth;
  final double itemHeight;

  SaifuQR(
      {Key? key,
      required this.data,
      this.charPerFrame = 100,
      this.splitData = false,
      this.errorCorrectionalLevel = QrErrorCorrectLevel.L,
      this.transitionDuration = 500,
      this.fade,
      this.scale = 1.0,
      this.itemHeight = 300,
      this.itemWidth = 300,
      this.loop = true,
      this.autoplay = true,
      this.autoplayDelay = 0,
      this.viewportFraction = 0.8,
      this.scrollDirection = Axis.horizontal,
      this.backgroundColor = Colors.white,
      this.foregroundColor = Colors.black,
      this.borderRadius = 10,
      this.qrPadding = 10})
      : super(key: key);
  @override
  State<SaifuQR> createState() => _SaifuQRState();
}

class _SaifuQRState extends State<SaifuQR> {
  var frameData;

  void handleData() {
    // Retrieves data and adds them togeether into a string
    String stringData = "";
    for (int i = 0; i < widget.data.length; i++) {
      stringData = stringData + widget.data[i].toString();
      setState(() {
        frameData = stringData;
      });
    }
    // Uses Regrex Expression to break the string of that previous data into equal parts by character length (charPerFrame)
    RegExp exp = RegExp(".{1," + widget.charPerFrame.toStringAsFixed(0) + "}");
    String str = frameData;
    Iterable<Match> matches = exp.allMatches(str);
    var list = matches.map((m) => m.group(0)).toList();

    // This is converted into a list, which is inputted into 1-1 on each individual QR
    setState(() {
      widget.data = list;
    });
  }

  @override
  void initState() {
    // If splitData is enabled, the user has decided to break the data into chunks, or else, the provided data is not split and processed 1-1 into the QR
    widget.splitData ? handleData() : null;

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final customLayoutOption = CustomLayoutOption(startIndex: 0, stateCount: 5);
    if (widget.data.isNotEmpty) {
      return Swiper(
        itemHeight: widget.itemHeight,
        itemWidth: widget.itemWidth,
        itemCount: widget.data.length,
        fade: widget.fade,
        scale: widget.scale,
        loop: widget.loop,
        autoplay: widget.autoplay,
        duration: widget.transitionDuration.toInt(),
        viewportFraction: widget.viewportFraction,
        autoplayDelay: widget.autoplayDelay,
        scrollDirection: widget.scrollDirection,
        layout: SwiperLayout.CUSTOM,
        customLayoutOption: customLayoutOption,
        itemBuilder: (BuildContext context, int index) {
          return Center(
            child: Container(
              alignment: Alignment.center,
              decoration: BoxDecoration(
                color: widget.backgroundColor,
                shape: BoxShape.rectangle,
                borderRadius: BorderRadius.all(Radius.circular(widget.borderRadius.toDouble())),
              ),
              child: Padding(
                padding: EdgeInsets.all(widget.qrPadding.toDouble()),
                child: QrImage(
                    foregroundColor: widget.foregroundColor,
                    errorCorrectionLevel: widget.errorCorrectionalLevel,
                    data: widget.data[index].toString(),
                    version: QrVersions.auto,
                    errorStateBuilder: (cxt, err) {
                      return const Center(
                        child: Text(
                          "QR code could not be generated",
                          textAlign: TextAlign.center,
                        ),
                      );
                    }),
              ),
            ),
          );
        },
      );
    } else {
      return Column(
        mainAxisSize: MainAxisSize.min,
        children: const [CircularProgressIndicator(), Text('Data List provided is null or empty')],
      );
    }
  }
}
